@@ -43,7 +43,8 @@ from obs.model import GetObjectHeader
 from obs.model import CopyObjectHeader
 from obs.model import SetObjectMetadataHeader
 from obs.bucket import BucketClient
-
+from obs import loadtoken
+from inspect import isfunction
 
 if const.IS_PYTHON2:
     from urlparse import urlparse
@@ -166,12 +167,13 @@ class ConvertWrapper(object):
 
 
 class _BasicClient(object):
-    def __init__(self, access_key_id, secret_access_key, is_secure=True, server=None,
+    def __init__(self, access_key_id='', secret_access_key='', is_secure=True, server=None,
                  signature='obs', region='region', path_style=False, ssl_verify=False,
                  port=None, max_retry_count=3, timeout=60, chunk_size=65536,
                  long_conn_mode=False, proxy_host=None, proxy_port=None,
                  proxy_username=None, proxy_password=None, security_token=None,
-                 custom_ciphers=None, use_http2=False, is_signature_negotiation=True, is_cname=False):
+                 custom_ciphers=None, use_http2=False, is_signature_negotiation=True, is_cname=False,
+                 max_redirect_count=10, security_providers=None, security_provider_policy=None):
         self.securityProvider = _SecurityProvider(access_key_id, secret_access_key, security_token)
         server = server if server is not None else ''
         server = util.to_string(util.safe_encode(server))
@@ -193,6 +195,24 @@ class _BasicClient(object):
         if len(host_port) == 2:
             port = util.to_int(host_port[1])
 
+        self.security_provider_policy = security_provider_policy
+        
+        if security_providers is None:
+            self.security_providers = [loadtoken.ENV, loadtoken.ECS]
+        else:
+            self.security_providers = security_providers    
+
+        try:
+            if security_providers == []:
+                raise ValueError('no available security_providers')
+            for method in self.security_providers:
+                getattr(method, '__name__')
+                if not isfunction(method.search):
+                    raise AttributeError(method+'has no function called search')
+        except Exception:
+            self.security_provider_policy = None
+            print(traceback.format_exc())        
+
         self.is_secure = is_secure
         self.server = host_port[0]
 
@@ -212,7 +232,8 @@ class _BasicClient(object):
         self.use_http2 = use_http2
         self.is_signature_negotiation = is_signature_negotiation
         self.is_cname = is_cname
-        
+        self.max_redirect_count = max_redirect_count
+
         if self.path_style or self.is_cname:
             self.is_signature_negotiation = False
             if self.signature == 'obs':
@@ -247,6 +268,20 @@ class _BasicClient(object):
             self.ha = convertor.Adapter(self.signature)
             self.convertor = convertor.Convertor(self.signature, self.ha)
 
+    def _get_token(self):
+        from obs.searchmethod import get_token
+        try:
+            if self.security_provider_policy is not None:
+                if self.securityProvider.access_key_id!='' and self.securityProvider.secret_access_key!='':
+                    return self.securityProvider                
+
+                value_dict = get_token(self.security_providers, name=self.security_provider_policy)
+                securityProvider = _SecurityProvider(value_dict.get('accessKey'),value_dict.get('secretKey'),value_dict.get('securityToken')) 
+                return securityProvider
+        except Exception:
+            self.log_client.log(WARNING, traceback.format_exc())                      
+        return self.securityProvider
+
     def _init_connHolder(self):
         if const.IS_PYTHON2:
             from Queue import Queue
@@ -332,14 +367,23 @@ class _BasicClient(object):
     def _make_put_request(self, bucketName, objectKey=None, pathArgs=None, headers=None, entity=None, chunkedMode=False, methodName=None, readable=False):
         return self._make_request_with_retry(const.HTTP_METHOD_PUT, bucketName, objectKey, pathArgs, headers, entity, chunkedMode, methodName=methodName, readable=readable)
 
+    def _make_error_result(self, e, ret):
+        self.log_client.log(ERROR, 'request error, %s' % e)
+        self.log_client.log(ERROR, traceback.format_exc())
+        if ret is not None:
+            return ret
+        raise e
+
     def _make_request_with_retry(self, methodType, bucketName, objectKey=None, pathArgs=None, headers=None, 
                        entity=None, chunkedMode=False, methodName=None, readable=False, parseMethod=None, redirectLocation=None, skipAuthentication=False):
         flag = 0
+        redirect_count = 0
         conn = None
         _redirectLocation = redirectLocation
+        redirectFlag = False
         while True:
             try:
-                conn = self._make_request_internal(methodType, bucketName, objectKey, pathArgs, headers, entity, chunkedMode, _redirectLocation, skipAuthentication=skipAuthentication)
+                conn = self._make_request_internal(methodType, bucketName, objectKey, pathArgs, headers, entity, chunkedMode, _redirectLocation, skipAuthentication=skipAuthentication, redirectFlag=redirectFlag)
                 return self._parse_xml(conn, methodName, readable) if not parseMethod else parseMethod(conn)
             except Exception as e:
                 ret = None
@@ -349,16 +393,20 @@ class _BasicClient(object):
                 else:
                     util.close_conn(conn, self.log_client)
                     if isinstance(e, _RedirectException):
+                        redirect_count += 1
                         _redirectLocation = e.location
                         flag -= 1
                         ret = e.result
-                
+                        if methodType == const.HTTP_METHOD_GET and  e.result.status == 302:
+                            redirectFlag = True
+                        else:
+                            redirectFlag = False
+                if redirect_count >= self.max_redirect_count:
+                    self.log_client.log(ERROR, 'request redirect count [%d] greater than max redirect count [%d]' % (
+                    redirect_count, self.max_redirect_count))
+                    return self._make_error_result(e, ret)
                 if flag >= self.max_retry_count or readable:
-                    self.log_client.log(ERROR, 'request error, %s' % e)
-                    self.log_client.log(ERROR, traceback.format_exc())
-                    if ret is not None:
-                        return ret
-                    raise e
+                    return self._make_error_result(e, ret)
                 flag += 1
                 time.sleep(math.pow(2, flag) * 0.05)
                 self.log_client.log(WARNING, 'request again, time:%d' % int(flag))
@@ -366,18 +414,26 @@ class _BasicClient(object):
             break
 
     def _make_request_internal(self, method, bucketName='', objectKey=None, pathArgs=None, headers=None, entity=None,
-                               chunkedMode=False, redirectLocation=None, skipAuthentication=False):
+                               chunkedMode=False, redirectLocation=None, skipAuthentication=False, redirectFlag=False):
         objectKey = util.safe_encode(objectKey)
         if objectKey is None:
             objectKey = ''
         port = None
         scheme = None
+        path = None
         if redirectLocation:
             redirectLocation = urlparse(redirectLocation)
             connect_server = redirectLocation.hostname
             scheme = redirectLocation.scheme
             port = redirectLocation.port if redirectLocation.port is not None else const.DEFAULT_INSECURE_PORT if scheme.lower() == 'http' else const.DEFAULT_SECURE_PORT
             redirect = True
+            _path = redirectLocation.path
+            query = redirectLocation.query
+            path = _path + '?' + query if query else _path
+            skipAuthentication = True
+            if not redirectFlag and not path:
+                skipAuthentication = False
+
         else:
             connect_server = self.server if self.is_cname else self.calling_format.get_server(self.server, bucketName)
             redirect = False
@@ -385,8 +441,10 @@ class _BasicClient(object):
         
         if self.is_cname:
             bucketName = ''
-            
-        path = self.calling_format.get_url(bucketName, objectKey, pathArgs)
+        
+        if not path:    
+            path = self.calling_format.get_url(bucketName, objectKey, pathArgs)
+        
         headers = self._rename_request_headers(headers, method)
 
         if entity is not None and not callable(entity):
@@ -399,7 +457,6 @@ class _BasicClient(object):
 
         headers[const.HOST_HEADER] = '%s:%s' % (connect_server, port) if port != 443 and port != 80 else connect_server
         header_config = self._add_auth_headers(headers, method, bucketName, objectKey, pathArgs, skipAuthentication)
-            
 
         header_log = header_config.copy()
         header_log[const.HOST_HEADER] = '******'
@@ -410,7 +467,7 @@ class _BasicClient(object):
 
     def _add_auth_headers(self, headers, method, bucketName, objectKey, pathArgs, skipAuthentication=False):
         from datetime import datetime
-        
+
         now_date = None
         if self.ha.date_header() not in headers:
             now_date = datetime.utcnow()
@@ -418,11 +475,11 @@ class _BasicClient(object):
         
         if skipAuthentication:
             return headers  
-        
-        securityProvider = self.securityProvider
+
+        securityProvider = self._get_token()
         ak = securityProvider.access_key_id
         sk = securityProvider.secret_access_key
-        
+
         if util.is_valid(ak) and util.is_valid(sk):
             if securityProvider.security_token is not None:
                 headers[self.ha.security_token_header()] = securityProvider.security_token
@@ -572,8 +629,7 @@ class _BasicClient(object):
         except _InternalException as ex:
             raise ex
         except Exception as e:
-            if self._need_clear(e):
-                conn._clear = True
+            conn._clear = True  
             self.log_client.log(ERROR, traceback.format_exc())
             raise e
         finally:
@@ -614,7 +670,7 @@ class _BasicClient(object):
             reason = result.reason
             self.convertor.parseGetObject(headers, body)
             header = self._rename_response_headers(headers)
-            requestId = headers.get(self.ha.request_id_header())
+            requestId = dict(header).get('request-id')
             return GetResult(status=status, reason=reason, header=header, body=body, requestId=requestId)
         except _RedirectException as ex:
             raise ex
@@ -676,7 +732,7 @@ class _BasicClient(object):
             reason = result.reason
             self.convertor.parseGetObject(headers, body)
             header = self._rename_response_headers(headers)
-            requestId = headers.get(self.ha.request_id_header())
+            requestId = dict(header).get('request-id')
             return GetResult(status=status, reason=reason, header=header, body=body, requestId=requestId)
         except _RedirectException as ex:
             raise ex
@@ -707,15 +763,15 @@ class _BasicClient(object):
                 readed_count += len(chunk)
         return origin_file_path, readed_count
     
-    def _rename_key(self, k, v, header_prefix, meta_header_prefix):
+    def _rename_key(self, k, v):
         flag = 0
-        if k.startswith(meta_header_prefix):
-            k = k[k.index(meta_header_prefix) + len(meta_header_prefix):]
+        if k.startswith(const.V2_META_HEADER_PREFIX):
+            k = k[k.index(const.V2_META_HEADER_PREFIX) + len(const.V2_META_HEADER_PREFIX):]
             k = util.decode_item(k)
             v = util.decode_item(v)
             flag = 1
-        elif k.startswith(header_prefix):
-            k = k[k.index(header_prefix) + len(header_prefix):]
+        elif k.startswith(const.V2_HEADER_PREFIX):
+            k = k[k.index(const.V2_HEADER_PREFIX) + len(const.V2_HEADER_PREFIX):]
             v = util.decode_item(v)
             flag = 1
         elif k.startswith(const.OBS_META_HEADER_PREFIX):
@@ -731,14 +787,12 @@ class _BasicClient(object):
 
     def _rename_response_headers(self, headers):
         header = []
-        header_prefix = self.ha._get_header_prefix()
-        meta_header_prefix = self.ha._get_meta_header_prefix()
         for k, v in headers.items():
             flag = 0
             if k in const.ALLOWED_RESPONSE_HTTP_HEADER_METADATA_NAMES:
                 flag = 1
             else:
-                flag, k, v = self._rename_key(k, v, header_prefix, meta_header_prefix)
+                flag, k, v = self._rename_key(k, v)
             if flag:
                 header.append((k, v))
         return header
@@ -755,7 +809,6 @@ class _BasicClient(object):
         headers = {}
         for k, v in result.getheaders():
             headers[k.lower()] = v
-        
         xml = None
         while True:
             chunk = result.read(chuckSize)
@@ -763,6 +816,7 @@ class _BasicClient(object):
                 break
             xml = chunk if xml is None else xml + chunk
         header = self._rename_response_headers(headers)
+        indicator = headers.get(self.ha.indicator_header())
         if status < 300:
             if methodName is not None:
                 parseMethod = getattr(self.convertor, 'parse' + methodName[:1].upper() + methodName[1:])
@@ -780,7 +834,9 @@ class _BasicClient(object):
                         self.log_client.log(ERROR, util.to_string(e))
                         self.log_client.log(ERROR, traceback.format_exc())
                     
-            requestId = headers.get(self.ha.request_id_header())
+            requestId = headers.get('x-obs-request-id')
+            if requestId is None:
+                requestId = headers.get('x-amz-request-id')
         elif xml:
             xml = xml if const.IS_PYTHON2 else xml.decode('UTF-8')
             try:
@@ -791,13 +847,20 @@ class _BasicClient(object):
                 self.log_client.log(ERROR, util.to_string(ee))
                 self.log_client.log(ERROR, traceback.format_exc())
                 
-        if not requestId:
-            requestId = headers.get(self.ha.request_id_header())
-            
+        if requestId is None:
+            requestId = headers.get('x-obs-request-id')
+        if requestId is None:
+            requestId = headers.get('x-amz-request-id')
+          
         self.log_client.log(DEBUG, 'http response result:status:%d,reason:%s,code:%s,message:%s,headers:%s',
                             status, reason, code, message, header)
+
+        if status >= 300:
+            self.log_client.log(ERROR, 'exceptional obs response:status:%d,reason:%s,code:%s,message:%s,requestId:%s',
+                                status, reason, code, message, requestId)
+
         ret = GetResult(code=code, message=message, status=status, reason=reason, body=body, 
-                         requestId=requestId, hostId=hostId, resource=resource, header=header)
+                         requestId=requestId, hostId=hostId, resource=resource, header=header, indicator=indicator)
         
         if not readable:
             if status >= 300 and status < 400 and status != 304 and const.LOCATION_HEADER.lower() in headers:
@@ -865,7 +928,7 @@ class ObsClient(_BasicClient):
         
         expires += util.to_int(time.time())
 
-        securityProvider = self.securityProvider
+        securityProvider = self._get_token()
         if securityProvider.security_token is not None and self.ha.security_token_header() not in queryParams:
             queryParams[self.ha.security_token_header()] = securityProvider.security_token
 
@@ -891,7 +954,7 @@ class ObsClient(_BasicClient):
         from datetime import datetime
 
         headers, queryParams, expires, calling_format = self._prepareParameterForSignedUrl(specialParam, expires, headers, queryParams)
-        
+
         if self.is_cname:
             connect_server = self.server
             bucketName = None
@@ -905,7 +968,7 @@ class ObsClient(_BasicClient):
         shortDate = date.strftime(const.SHORT_DATE_FORMAT)
         longDate = date.strftime(const.LONG_DATE_FORMAT)
 
-        securityProvider = self.securityProvider
+        securityProvider = self._get_token()
 
         if securityProvider.security_token is not None and self.ha.security_token_header() not in queryParams:
             queryParams[self.ha.security_token_header()] = securityProvider.security_token
@@ -916,7 +979,7 @@ class ObsClient(_BasicClient):
         queryParams['X-Amz-Credential'] = v4Auth.getCredenttial()
         queryParams['X-Amz-Date'] = longDate
         queryParams['X-Amz-Expires'] = expires
-        
+
         headMap = v4Auth.setMapKeyLower(headers)
         signedHeaders = v4Auth.getSignedHeaders(headMap)
         
@@ -948,7 +1011,7 @@ class ObsClient(_BasicClient):
         date = datetime.utcnow()
         shortDate = date.strftime(const.SHORT_DATE_FORMAT)
         longDate = date.strftime(const.LONG_DATE_FORMAT)
-        securityProvider = self.securityProvider
+        securityProvider = self._get_token()
 
         expires = 300 if expires is None else util.to_int(expires)
         expires = date + timedelta(seconds=expires)
@@ -1092,6 +1155,8 @@ class ObsClient(_BasicClient):
     def setBucketAcl(self, bucketName, acl=ACL(), aclControl=None):
         if acl is not None and len(acl) > 0 and aclControl is not None:
             raise Exception('Both acl and aclControl are set')
+        if not acl and not aclControl:
+            raise Exception('Both acl and aclControl are not set')
         return self._make_put_request(bucketName, **self.convertor.trans_set_bucket_acl(acl=acl, aclControl=aclControl))
 
     @funcCache
@@ -1273,7 +1338,7 @@ class ObsClient(_BasicClient):
             content = AppendObjectContent()
             
         if headers.get('contentType') is None:
-            headers['contentType'] = const.MIME_TYPES.get(objectKey[objectKey.rfind('.') + 1:])
+            headers['contentType'] = const.MIME_TYPES.get(objectKey[objectKey.rfind('.') + 1:].lower())
         
         chunkedMode = False
         readable = False
@@ -1286,7 +1351,7 @@ class ObsClient(_BasicClient):
                     raise Exception('file [%s] does not exist' % file_path)
             
             if headers.get('contentType') is None:
-                headers['contentType'] = const.MIME_TYPES.get(file_path[file_path.rfind('.') + 1:])
+                headers['contentType'] = const.MIME_TYPES.get(file_path[file_path.rfind('.') + 1:].lower())
             
             file_size = util.to_long(os.path.getsize(file_path))
             headers['contentLength'] = util.to_long(headers.get('contentLength'))
@@ -1347,7 +1412,7 @@ class ObsClient(_BasicClient):
         if headers is None:
             headers = PutObjectHeader()
         if headers.get('contentType') is None:
-            headers['contentType'] = const.MIME_TYPES.get(objectKey[objectKey.rfind('.') + 1:])
+            headers['contentType'] = const.MIME_TYPES.get(objectKey[objectKey.rfind('.') + 1:].lower())
         _headers = self.convertor.trans_put_object(metadata=metadata, headers=headers)
         
         readable = False
@@ -1419,10 +1484,10 @@ class ObsClient(_BasicClient):
             headers['contentLength'] = size if headers['contentLength'] > size else headers['contentLength']
 
         if headers.get('contentType') is None:
-            headers['contentType'] = const.MIME_TYPES.get(objectKey[objectKey.rfind('.') + 1:])
+            headers['contentType'] = const.MIME_TYPES.get(objectKey[objectKey.rfind('.') + 1:].lower())
 
         if headers.get('contentType') is None:
-            headers['contentType'] = const.MIME_TYPES.get(file_path[file_path.rfind('.') + 1:])
+            headers['contentType'] = const.MIME_TYPES.get(file_path[file_path.rfind('.') + 1:].lower())
 
         _headers = self.convertor.trans_put_object(metadata=metadata, headers=headers)
         if const.CONTENT_LENGTH_HEADER not in _headers:
@@ -1606,6 +1671,8 @@ class ObsClient(_BasicClient):
     def setObjectAcl(self, bucketName, objectKey, acl=ACL(), versionId=None, aclControl=None):
         if acl is not None and len(acl) > 0 and aclControl is not None:
             raise Exception('Both acl and aclControl are set')
+        if not acl and not aclControl:
+            raise Exception('Both acl and aclControl are not set')
         return self._make_put_request(bucketName, objectKey, **self.convertor.trans_set_object_acl(acl=acl, versionId=versionId, aclControl=aclControl))
 
 
@@ -1642,7 +1709,7 @@ class ObsClient(_BasicClient):
             objectKey = ''
         
         if contentType is None:
-            contentType = const.MIME_TYPES.get(objectKey[objectKey.rfind('.') + 1:])
+            contentType = const.MIME_TYPES.get(objectKey[objectKey.rfind('.') + 1:].lower())
         
         return self._make_post_request(bucketName, objectKey, methodName='initiateMultipartUpload', 
                                        **self.convertor.trans_initiate_multipart_upload(acl=acl, storageClass=storageClass, 
@@ -1720,9 +1787,9 @@ class ObsClient(_BasicClient):
     
 
     @funcCache
-    def uploadFile(self, bucketName, objectKey, uploadFile, partSize=5 * 1024 * 1024, 
+    def uploadFile(self, bucketName, objectKey, uploadFile, partSize=9 * 1024 * 1024, 
                    taskNum=1, enableCheckpoint=False, checkpointFile=None, 
-                   checkSum=False, metadata=None, progressCallback=None):
+                   checkSum=False, metadata=None, progressCallback=None, headers=None):
         self.log_client.log(INFO, 'enter resume upload file...')
         self._assert_not_null(bucketName, 'bucketName is empty')
         self._assert_not_null(objectKey, 'objectKey is empty')
@@ -1740,7 +1807,7 @@ class ObsClient(_BasicClient):
             taskNum = 1
         else:
             taskNum = int(math.ceil(taskNum))
-        return _resumer_upload(bucketName, objectKey, uploadFile, partSize, taskNum, enableCheckpoint, checkpointFile, checkSum, metadata, progressCallback, self)
+        return _resumer_upload(bucketName, objectKey, uploadFile, partSize, taskNum, enableCheckpoint, checkpointFile, checkSum, metadata, progressCallback, self, headers)
     
     @funcCache
     def _downloadFileWithNotifier(self, bucketName, objectKey, downloadFile=None, partSize=5 * 1024 * 1024, taskNum=1, enableCheckpoint=False,
