@@ -110,7 +110,8 @@ __all__ = [
     'UploadPartResponse',
     'ResponseWrapper',
     'ObjectStream',
-    'GetBucketEncryptionResponse'
+    'GetBucketEncryptionResponse',
+    'UploadFileHeader'
 ]
 
 
@@ -141,9 +142,10 @@ class BaseModel(dict):
 class GetResult(BaseModel):
  
     allowedAttr = {'status': int, 'reason':BASESTRING, 'errorCode': BASESTRING, 'errorMessage': BASESTRING,
-                   'body': object, 'requestId': BASESTRING, 'hostId': BASESTRING, 'resource': BASESTRING, 'header':list}
+                   'body': object, 'requestId': BASESTRING, 'hostId': BASESTRING, 'resource': BASESTRING, 'header':list,
+                   'indicator': BASESTRING}
  
-    def __init__(self, code=None, message=None, status=None, reason=None, body=None, requestId=None, hostId=None, resource=None, header=None):
+    def __init__(self, code=None, message=None, status=None, reason=None, body=None, requestId=None, hostId=None, resource=None, header=None, indicator=None):
         self.status = status
         self.reason = reason
         self.errorCode = code
@@ -153,6 +155,7 @@ class GetResult(BaseModel):
         self.hostId = hostId
         self.resource = resource
         self.header = header
+        self.indicator = indicator
 
 class CompletePart(BaseModel):
     allowedAttr = {'partNum': int, 'etag': BASESTRING}
@@ -616,6 +619,21 @@ class PutObjectHeader(BaseModel):
 
 AppendObjectHeader = PutObjectHeader
 
+class UploadFileHeader(BaseModel):
+    allowedAttr = {'acl': BASESTRING, 'websiteRedirectLocation': BASESTRING,'contentType': BASESTRING, 'sseHeader': SseHeader,
+                   'storageClass': BASESTRING, 'successActionRedirect': BASESTRING, 'expires': int, 'extensionGrants': list}
+
+    def __init__(self, acl=None, websiteRedirectLocation=None, contentType=None, sseHeader=None,
+                 storageClass=None, successActionRedirect=None, expires=None, extensionGrants=None):
+        self.acl = acl
+        self.websiteRedirectLocation = websiteRedirectLocation
+        self.contentType = contentType
+        self.sseHeader = sseHeader
+        self.storageClass = storageClass
+        self.successActionRedirect = successActionRedirect
+        self.expires = expires
+        self.extensionGrants = extensionGrants
+
 class AppendObjectContent(BaseModel):
     allowedAttr = {'content': [object], 'position': [LONG, int, BASESTRING], 'offset':[LONG, int, BASESTRING], 'isFile': bool}
     def __init__(self, content=None, position=None, offset=None, isFile=False):
